/****************************************************************\
*                                                                *
*  Comparison : A module for pairwise sequence comparisons       *
*                                                                *
*  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
*  Copyright (C) 2000-2006.  All Rights Reserved.                *
*                                                                *
*  This source code is distributed under the terms of the        *
*  GNU Lesser General Public License. See the file COPYING       *
*  or http://www.fsf.org/copyleft/lesser.html for details        *
*                                                                *
*  If you use this code, please keep this notice intact.         *
*                                                                *
\****************************************************************/

#ifndef INCLUDED_COMPARISON_H
#define INCLUDED_COMPARISON_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <glib.h>
#include "argument.h"
#include "alphabet.h"
#include "sequence.h"
#include "translate.h"
#include "submat.h"
#include "hspset.h"

typedef struct Comparison_Param {
                      gint  ref_count;
             Alphabet_Type  query_type;
             Alphabet_Type  target_type;
                 HSP_Param *dna_hsp_param;
                 HSP_Param *protein_hsp_param;
                 HSP_Param *codon_hsp_param;
   struct Comparison_Param *mirror;
} Comparison_Param;

Comparison_Param *Comparison_Param_create(Alphabet_Type query_type,
                                          Alphabet_Type target_type,
                                          HSP_Param *dna_hsp_param,
                                          HSP_Param *protein_hsp_param,
                                          HSP_Param *codon_hsp_param);
void Comparison_Param_destroy(Comparison_Param *param);
Comparison_Param *Comparison_Param_share(Comparison_Param *param);
Comparison_Param *Comparison_Param_swap(Comparison_Param *param);

/**/

typedef struct {
                gint  ref_count;
    Comparison_Param *param;
            Sequence *query;
            Sequence *target;
              HSPset *dna_hspset;
              HSPset *protein_hspset;
              HSPset *codon_hspset;
} Comparison;

Comparison *Comparison_create(Comparison_Param *param,
                              Sequence *query, Sequence *target);
Comparison *Comparison_share(Comparison *comparison);
void Comparison_destroy(Comparison *comparison);

gboolean Comparison_has_hsps(Comparison *comparison);
    void Comparison_finalise(Comparison *comparison);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* INCLUDED_COMPARISON_H */

